import re
import os
import urllib.parse
#from logging import Logger

import lxml.html

from craigsgenerator.cache import get
from craigsgenerator.parse import search_row, listing

#logger = Logger('craigsgenerator')

class Section:
    def __init__(self, subdomain, section, *args, cachedir = 'craigslist', scheme = 'https', **kwargs):
        if scheme not in {'http','https'}:
            raise ValueError('Scheme must be one of "http" or "https".')

        self.subdomain = subdomain
        self.section = section
        self.cachedir = cachedir
        self.args = args
        self.kwargs = kwargs
        self.scheme = scheme

    def __iter__(self):
        self.buffer = []
        self.html = None
        self.present_search_url = None
        return self

    def __next__(self):
        if self.buffer == []:
            self.download()
            self.buffer.extend(map(search_row,self.html.xpath('//p[@class="row"]')))

        if self.html.xpath('count(//p[@class="row"])') == 0:
           #logger.debug('Stopped at %s' % self.present_search_url)
            raise StopIteration
        else:
            row = self.buffer.pop(0)
            row['listing'] = get(self.cachedir, row['href'],
                                 False, *self.args, **self.kwargs)
            return row

    def download(self):
        self.present_search_url = self.next_search_url()
        fp = get(self.cachedir, self.present_search_url, True, *self.args, **self.kwargs)

        html = lxml.html.fromstring(fp.read())
        html.make_links_absolute(self.present_search_url)
        self.html = html

    def next_search_url(self):
        'Determine the url of the next search page.'
        if not self.html:
            return '%s://%s.craigslist.org/%s/index000.html' % (self.scheme, self.subdomain, self.section)

        nexts = set(self.html.xpath('//a[contains(text(),"next >")]/@href'))
        if len(nexts) != 1:
            raise ValueError('No next page for %s' % self.search_url)
        return str(list(nexts)[0])

def subdomains(url = 'https://sfbay.craigslist.org', cachedir = 'craigslist', id = 'rightbar'):
    results = set()
    fp = get(cachedir, url, False)
    html = lxml.html.fromstring(fp.read())
    for href in html.xpath('id("%s")/descendant::a/@href' % id):
        p = urllib.parse.urlparse(href.rstrip('/'))
        if p.fragment:
            pass
        elif p.path:
            results.update(subdomains(url = href, cachedir = cachedir, id = 'list'))
        else:
            print(p.netloc)
            results.add(p.netloc)
    return results

def fulltext(row):
    'Given a row from the Section generator, produce the full text of the listing.'
    html = lxml.html.fromstring(row['listing'].read())
    return listing(html)



import lxml.html

import craigsgenerator.parse as parse

p = lxml.html.fromstring('''
 <p class="row" data-latitude="30.289600" data-longitude="-97.739600" data-pid="4331364900"> <a href="/sub/4331364900.html" class="i"></a> <span class="star"></span> <span class="pl"> <span class="date">Feb 15</span>  <a href="/sub/4331364900.html">Pre-leasing Apt in WEST CAMPUS for Summer 2014</a> </span> <span class="l2">  <span class="price">&#x0024;925</span> / 1br -  <span class="pnr"> <small> (806 West 24th St)</small> <span class="px"> <span class="p"> <a href="#" class="maptag" data-pid="4331364900">map</a></span></span> </span>  </span> </p>
''')
p.make_links_absolute('https://austin.craigslist.org/sub/index200.html')

def test_search_row_with_location():
    o = parse.search_row(p)
    e = {
        'href': 'https://austin.craigslist.org/sub/4331364900.html',
        'latitude': 30.289600,
        'longitude': -97.739600,
        'date': 'Feb 15',
        'title': 'Pre-leasing Apt in WEST CAMPUS for Summer 2014',
        'price': 925,
    }
    assert o == e



SEARCH_ROW_ATTRS = [
    ('href','a/@href', str),
    ('latitude', '@data-latitude', float),
    ('longitude', '@data-longitude', float),
    ('date','span[@class="pl"]/span[@class="date"]/text()', str),
    ('title','span[@class="pl"]/a/text()', str),
    ('price','span[@class="l2"]/span[@class="price"]/text()', lambda x: int(x.strip('$',))),
]

def search_row(p):
    'Parse a <p class="row"></p>.'
    row = {}
    for key, attr, func in SEARCH_ROW_ATTRS:
        results = p.xpath(attr)
        if len(results) == 1:
            row[key] = func(results[0])

    ('href','a/@href'),
    ('latitude', '@data-latitude'),
    ('longitude', '@data-longitude'),
    ('date','span[@class="pl"]/span[@class="date"]/text()'),
    ('title','span[@class="pl"]/a/text()'),
    ('price','span[@class="l2"]/span[@class="price"]/text()'),

    return row

def listing(html):
    postingbody = html.get_element_by_id('postingbody')
    return postingbody.text_content()



import os
import datetime
import urllib.parse

import requests

def get(cachedir, url, refresh, *args, **kwargs):
    urldir = _url_to_directory(cachedir, url)
    if (cachedir == None) or (not os.path.exists(urldir)) or (os.listdir(urldir) == []):
        _download(cachedir, url, *args, **kwargs)

    mostrecent = sorted(filter(lambda f: os.path.isfile(os.path.join(urldir, f)), os.listdir(urldir)))[-1]
    if refresh and datetime.datetime.strptime(mostrecent, '%Y-%m-%d').date() < datetime.date.today():
        _download(cachedir, url, *args, **kwargs)

    return open(os.path.join(urldir, mostrecent), 'r')

def _download(cachedir, url, date = datetime.date.today(), *args, **kwargs):
    directory = _url_to_directory(cachedir, url)
    filename = os.path.join(directory, date.isoformat())
    if not os.path.exists(directory):
        os.makedirs(directory)

    r = requests.get(url, *args, **kwargs)
    with open(filename, 'x') as f:
        f.write(r.text)

def _url_to_directory(cachedir, url):
    p = urllib.parse.urlparse(url)
    if p.query or p.fragment or p.params:
        raise ValueError('query, fragment and params are not allowed.')
    return os.path.join(*([cachedir, p.netloc] + p.path.strip('/').split('/')))



from distutils.core import setup

#with open('README.rst') as file:
#    long_description = file.read()

setup(name='craigsgenerator',
    author='Thomas Levine',
    author_email='_@thomaslevine.com',
    description='Iterate through Craig\'s list',
#   long_description=long_description,
    url='https://github.com/tlevine/craigsgenerator.git',
    classifiers=[
        'Intended Audience :: Developers',
    ],
    packages=['craigsgenerator'],

    install_requires = ['lxml'],

    version='0.0.4',
    license='AGPL'
)
